<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access
defined('JPATH_BASE') or die;

// Load the base adapter.
require_once JPATH_ADMINISTRATOR . '/components/com_finder/helpers/indexer/adapter.php';

/**
 * Finder adapter for Joomla Tag.
 *
 * @package Joomla.Plugin
 * @subpackage RSDirectory!.Entries
 * @since 3.1
 */
class plgFinderRSDirectory extends FinderIndexerAdapter
{
	/**
	 * The plugin identifier.
	 *
	 * @access protected
	 *
	 * @var string
	 * 
	 * @since  3.1
	 */
	protected $context = 'Entries';
		
	/**
	 * The extension name.
	 *
	 * @access protected
	 *
	 * @var string
	 * 
	 * @since 3.1
	 */
	protected $extension = 'com_rsdirectory';
		
	/**
	 * The sublayout to use when rendering the results.
	 *
	 * @access protected
	 *
	 * @var string
	 * 
	 * @since 3.1
	 */
	protected $layout = 'entry';
		
	/**
	 * The type of content that the adapter indexes.
	 *
	 * @access protected
	 *
	 * @var string
	 * 
	 * @since 3.1
	 */
	protected $type_title = 'Entry';
		
	/**
	 * The table name.
	 *
	 * @access protected
	 *
	 * @var string
	 * 
	 * @since 3.1
	 */
	protected $table = '#__rsdirectory_entries';
		
	/**
	 * Load the language file on instantiation.
	 *
	 * @access protected
	 *
	 * @var boolean
	 * 
	 * @since  3.1
	 */
	protected $autoloadLanguage = true;
		
	/**
	 * The field the published state is stored in.
	 *
	 * @access protected
	 *
	 * @var string
	 * 
	 * @since 3.1
	 */
	protected $state_field = 'published';
		
	/**
	 * Method to remove the link information for items that have been deleted.
	 *
	 * @access public
	 *
	 * @param string $context The context of the action being performed.
	 * @param JTable $table A JTable object containing the record to be deleted
	 *
	 * @return boolean True on success.
	 *
	 * @since 3.1
	 * @throws Exception on database error.
	 */
	public function onFinderAfterDelete($context, $table)
	{
		if ($context == 'com_rsdirectory.entry')
		{
			$id = $table->id;
		}
		else if ($context == 'com_finder.index')
		{
			$id = $table->link_id;
		}
		else
		{
			return true;
		}
			
		// Remove the items.
		return $this->remove($id);
	}
		
	/**
	 * Method to determine if the access level of an item changed.
	 *
	 * @access public
	 *
	 * @param string $context The context of the content passed to the plugin.
	 * @param JTable $row A JTable object
	 * @param boolean $isNew If the content has just been created
	 *
	 * @return boolean True on success.
	 *
	 * @since 3.1
	 * @throws Exception on database error.
	 */
	public function onFinderAfterSave($context, $row, $isNew)
	{
		// We only want to handle tags here.
		if ($context == 'com_rsdirectory.entry')
		{
			// Check if the access levels are different.
			if (!$isNew && $this->old_access != $row->access)
			{
				// Process the change.
				$this->itemAccessChange($row);
			}
				
			// Reindex the item
			$this->reindex($row->id);
		}
			
		return true;
	}
		
	/**
	 * Method to reindex the link information for an item that has been saved.
	 * This event is fired before the data is actually saved so we are going
	 * to queue the item to be indexed later.
	 *
	 * @access public
	 *
	 * @param string $context The context of the content passed to the plugin.
	 * @param JTable $row A JTable object
	 * @param boolean $isNew If the content is just about to be created
	 *
	 * @return boolean  True on success.
	 *
	 * @since 3.1
	 * @throws Exception on database error.
	 */
	public function onFinderBeforeSave($context, $row, $isNew)
	{
		// We only want to handle news feeds here.
		if ($context == 'com_rsdirectory.entry')
		{
			// Query the database for the old access level if the item isn't new.
			if (!$isNew)
			{
				$this->checkItemAccess($row);
			}
		}
			
		return true;
	}
		
	/**
	 * Method to update the link information for items that have been changed
	 * from outside the edit screen. This is fired when the item is published,
	 * unpublished, archived, or unarchived from the list view.
	 *
	 * @access public
	 *
	 * @param string $context The context for the content passed to the plugin.
	 * @param array $pks A list of primary key ids of the content that has changed state.
	 * @param integer $value The value of the state that the content has been changed to.
	 *
	 * @return void
	 *
	 * @since 3.1
	 */
	public function onFinderChangeState($context, $pks, $value)
	{
		// We only want to handle tags here.
		if ($context == 'com_rsdirectory.entry')
		{
			$this->itemStateChange($pks, $value);
		}
			
		// Handle when the plugin is disabled
		if ($context == 'com_plugins.plugin' && $value === 0)
		{
			$this->pluginDisable($pks);
		}
	}

	/**
	 * Method to index an item. The item must be a FinderIndexerResult object.
	 *
	 * @access protected
	 *
	 * @param FinderIndexerResult $item The item to index as an FinderIndexerResult object.
	 * @param string $format The item format
	 *
	 * @return void
	 *
	 * @since 3.1
	 * @throws Exception on database error.
	 */
	protected function index(FinderIndexerResult $item, $format = 'html')
	{
		// Check if the extension is enabled and compatible with RSDirectory!
		if ( JComponentHelper::isEnabled($this->extension) == false || !method_exists('RSDirectoryRoute', 'getEntryURL') )
			return;
			
		// Build the necessary route and path information.
		$item->url = $this->getURL($item->id, $this->extension, $this->layout);
		$item->route = RSDirectoryRoute::getEntryURL($item->id, $item->title, '', 0, false, false);
			
		// Get the menu title if it exists.
		$title = $this->getItemMenuTitle($item->url);
			
		// Adjust the title if necessary.
		if ( !empty($title) && $this->params->get('use_menu_title', true) )
		{
			$item->title = $title;
		}
			
		// Add the meta-author.
		$item->metaauthor = $item->author;
			
		// Handle the link to the meta-data.
		$item->addInstruction(FinderIndexer::META_CONTEXT, 'link');
		$item->addInstruction(FinderIndexer::META_CONTEXT, 'author');
			
		// Add the type taxonomy data.
		$item->addTaxonomy('Type', 'Entries');
			
		// Add the author taxonomy data.
		if ( !empty($item->author) )
		{
			$item->addTaxonomy('Author', $item->author);
		}
			
		// Get content extras.
		FinderIndexerHelper::getContentExtras($item);
			
		// Index the item.
		$this->indexer->index($item);
		
	}
		
	/**
	 * Method to setup the indexer to be run.
	 *
	 * @access protected
	 *
	 * @return boolean True on success.
	 *
	 * @since 3.1
	 */
	protected function setup()
	{
		// Load dependent classes.
		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php';
		return true;
	}

	/**
	 * Method to get the SQL query used to retrieve the list of content items.
	 *
	 * @access protected
	 *
	 * @param mixed $query A JDatabaseQuery object or null.
	 *
	 * @return JDatabaseQuery A database object.
	 *
	 * @since 3.1
	 */
	protected function getListQuery($query = null)
	{
		// Get DBO.
		$db = JFactory::getDbo();
			
		$select = array(
			$db->qn('a.id'),
			$db->qn('a.title'),
			$db->qn('a.description', 'summary'),
			$db->qn('a.published_time', 'start_date'),
			$db->qn('a.user_id', 'created_by'),
			$db->q('1') . ' AS ' . $db->qn('access'),
			$db->qn('a.expiry_time', 'end_date'),
			$db->qn('a.published', 'state'),
			$db->qn('u.name', 'author'),
		);
			
		// Check if we can use the supplied SQL query.
		$query = $query instanceof JDatabaseQuery ? $query : $db->getQuery(true)
			  ->select($select)
			  ->from( $db->qn('#__rsdirectory_entries', 'a') )
			  ->innerJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('a.user_id') . ' = ' . $db->qn('u.id') );
				
		return $query;
	}
	
	/**
	 * Method to get a SQL query to load the published and access states for
	 * an article and category.
	 *
	 * @return  JDatabaseQuery  A database object.
	 *
	 * @since   2.5
	 */
	protected function getStateQuery() {
		$db		= JFactory::getDbo();
		$query  = $db->getQuery(true);

		// Item ID
		$query->select('a.id');
		$query->select('a.' . $this->state_field . ' AS state');
		$query->from($this->table . ' AS a');

		return $query;
	}
}