<?php
/**
 * @package RSDirectory!
 * @copyright  (c) 2013 - 2021 RSJoomla!
 * @link       https://www.rsjoomla.com
 * @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

// Load the RSDirectory! Plugin helper class file if it exists or stop the execution of the script if it doesn't exist.
if ( file_exists(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php') )
{
    require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php';
}
else
{
    return;
}

/**
 * RSDirectory! iDEAL Payment Plugin.
 */
class plgSystemRsdirectoryideal extends RSDirectoryPlugin
{
	protected $autoloadLanguage = true;

    protected static $payment_methods = array('mollie', 'sisow', 'targetpay');

    const PLUGIN = 'rsdirectoryideal';
    const EXTENSION = 'plg_system_rsdirectoryideal';
        
    /**
     * Class constructor.
     * 
     * @access public
     * 
     * @param object &$subject
     * @param array $config
     */
    public function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);

		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/iDEAL/Mollie/mollie.php';
		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/iDEAL/sisow.php';
		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/iDEAL/targetpay.php';
    }

	protected function getActiveServices()
	{
		static $active_services;

		if ($active_services === null)
		{
			$active_services = array();

			if ($this->params->get('use_mollie'))
			{
				$api = $this->params->get('mode') ? $this->params->get('mollieapi', '') : $this->params->get('mollieapi_test', '');
				if ($api)
				{
					$active_services[] = 'mollie';
				}
			}

			if ($this->params->get('use_sisow'))
			{
				$merchant_id = trim( $this->params->get('sisowmerchantid', ''));
				$merchant_key = trim( $this->params->get('sisowmerchantkey', ''));

				if ($merchant_id && $merchant_key)
				{
					$active_services[] = 'sisow';
				}
			}

			if ($this->params->get('use_targetpay'))
			{
				$layoutcode = trim( $this->params->get('targetpaylayoutcode', ''));

				if ($layoutcode)
				{
					$active_services[] = 'targetpay';
				}
			}
		}

		return $active_services;
	}

    /**
     * Can the plugin run?
     *
     * @access public
     * 
     * @return bool
     */
    public function canRun()
    {
        return parent::canRun() && $this->getActiveServices() && JPluginHelper::isEnabled('system', self::PLUGIN);
    }
        
    /**
     * On after render.
     *
     * @access public
     */
    public function onAfterRender()
    {
		if ( !$this->canRun() )
		{
			return;
		}

		if (JFactory::getApplication()->input->getInt('rsdirectoryidealpayment'))
		{
			$this->processForm();
		}
    }
        
    /**
     * Add the current payment option to the payments list.
     *
     * @access public
     * 
     * @return array|bool
     */
    public function onRsdirectoryaddPaymentOptions()
    {
		if ( !$this->canRun() )
			return false;

		// get the banks for the active services
		$options = array();

		if ($active_services = $this->getActiveServices())
		{
			$tax_text = '';

			if ( $tax_value = $this->params->get('tax_value') )
			{
				if ( $this->params->get('tax_type') )
				{
					$tax_text = "$tax_value%";
				}
				else
				{
					$tax_text = RSDirectoryHelper::formatPrice($tax_value);
				}
			}

			foreach ($active_services as $method)
			{
				$class = 'RSDirectory'.ucfirst($method);

				if (!class_exists($class))
				{
					continue;
				}

				try
				{
					$banks = $class::getInstance($this->params)->banks;

					$options[] = (object) array(
						'value' 	=> $method,
						'text' 		=> JText::_('PLG_SYSTEM_RSDIRECTORYIDEAL_' . $method . '_METHOD'),
						'tax_text' 	=> $tax_text,
						'banks' 	=> $banks
					);
				}
				catch (Exception $e)
				{
					JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
				}
			}
		}
			
		return $options;
    }
        
    /**
     * Show payment form.
     *
     * @access public
     * 
     * @param object $vars
     */
    public function onRsdirectoryshowForm($vars)
	{
		// Do a few checks and exit the function if the conditions are not met.
		if (!$this->canRun() || !isset($vars->method) || !in_array($vars->method, self::$payment_methods))
			return;

		// hold the redirect url for the specific service
		$redirect_url = '';

		$system_errors = array();
		switch ($vars->method) {
			case 'mollie':
				try
				{
					$ideal = RSDirectoryMollie::getInstance($this->params);
					$sale = &$ideal->sale;

					/**
					 * Build an array with parameters so we can
					 * overwrite it with arguments passed through
					 * the Scripts Called on Form Process
					 */
					$args = array(
						'amount'      => array(
							'value' 	=> number_format($vars->total, 2, '.', ''),
							'currency'	=> RSDirectoryConfig::getInstance()->get('currency')
						),
						'method' => \Mollie\Api\Types\PaymentMethod::IDEAL,
						'description' => $vars->item_name,
						'redirectUrl' => JUri::root() . 'index.php?option=com_rsdirectory&rsdirectoryidealpayment=1&task=plugin&hash=' . $vars->hash . '&rsdir_plugin_task=mollie.return',
						'webhookUrl' => JUri::root() . 'index.php?option=com_rsdirectory&rsdirectoryidealpayment=1&task=plugin&hash=' . $vars->hash . '&rsdir_plugin_task=mollie.notify',
						'metadata' => array(
							'order_id' => $vars->hash
						),
						'issuer' => $vars->payment_method_bank
					);

					$args = array_merge($args, $ideal->args);

					/**
					 * Create the Mollie Transaction Object,
					 * this will return a transaction ID that will
					 * be stored in the database
					 */
					$payment = $sale->payments->create($args);

					$db    = JFactory::getDbo();
					$query = $db->getQuery(true);
					$query->update($db->qn('#__rsdirectory_users_transactions'))
						->set($db->qn('gateway_order_number') . ' = ' . $db->q($payment->id))
						->where($db->qn('hash') . '=' . $db->q($vars->hash));
					$db->setQuery($query);
					$db->execute();

					$redirect_url = $payment->getCheckoutUrl();
				}
				catch (Exception $e)
				{
					$system_errors[] = $e->getMessage();
				}
			break;

			case 'sisow':
				try
				{
					/**
					 * Get an instance of the Sisow object (it might have been initiated during form process)
					 */
					$ideal = RSDirectorySisow::getInstance($this->params);
					$sale  = &$ideal->sale;

					/**
					 * Build an array with parameters so we can
					 * overwrite it with arguments passed through
					 * the Scripts Called on Form Process
					 */
					$args = array(
						'amount'      => number_format($vars->total, 2, '.', ''),
						'description' => substr($vars->item_name, 0, 32),
						'purchaseId'  => substr($vars->hash, 0, 16),
						'issuerId'    => $vars->payment_method_bank,
						'returnUrl'   => JURI::root() . 'index.php?option=com_rsdirectory&rsdirectoryidealpayment=1&task=plugin&hash=' . $vars->hash . '&rsdir_plugin_task=sisow.notify'
					);

					$args  = array_merge($args, $ideal->args);

					/**
					 * Build the Sisow Request object
					 */
					foreach ($args as $key => $value)
					{
						$sale->{$key} = $value;
					}

					/**
					 * Send the request, it will provide the $sale->issueUrl
					 */
					if (($payment = $sale->TransactionRequest()) < 0)
					{
						throw new Exception ($sale->errorCode . ' ' . $sale->errorMessage);
					}

					/**
					 * Redirect to the payment gateway
					 */
					$redirect_url = $sale->issuerUrl;
				}
				catch (Exception $e)
				{
					$system_errors[] = $e->getMessage();
				}
			break;

			case 'targetpay':
				try
				{
					/**
					 * Get an instance of the Sisow object (it might have been initiated during form process)
					 */
					$ideal = RSDirectoryTargetPay::getInstance($this->params);
					$sale  = &$ideal->sale;
					
					// for TargetPay the total needs to be in cents
					$total = $vars->total * 100;
					/**
					 * Build an array with parameters so we can
					 * overwrite it with arguments passed through
					 * the Scripts Called on Form Process
					 */
					$args = array(
						'bank'        => $vars->payment_method_bank,
						'description' => $vars->item_name,
						'amount'      => $total,
						'returnurl'   => JURI::root() . 'index.php?option=com_rsdirectory&rsdirectoryidealpayment=1&task=plugin&hash=' . $vars->hash . '&rsdir_plugin_task=targetpay.notify',
						'ver'		  => 3
					);
					
					$args  = array_merge($args, $ideal->args);

					/**
					 * Send the request, it will provide the $sale->url
					 */
					$sale->TransactionRequest($args);

					/**
					 * Create the Mollie Transaction Object,
					 * this will return a transaction ID that will
					 * be stored in the database
					 */
					$db    = JFactory::getDbo();
					$query = $db->getQuery(true);
					$query->update($db->qn('#__rsdirectory_users_transactions'))
						->set($db->qn('gateway_order_number') . ' = ' . $db->q($sale->trxid))
						->where($db->qn('hash') . '=' . $db->q($vars->hash));
					$db->setQuery($query);
					$db->execute();

					/**
					 * Redirect to the payment gateway
					 */
					$redirect_url = $sale->url;
				}
				catch (Exception $e)
				{
					$system_errors[] = $e->getMessage();
				}
			break;
		}

		self::outputTransactionDetails($vars);

		if (!empty($system_errors))
		{
			$errors = implode('<br/>', $system_errors);
			JFactory::getApplication()->enqueueMessage( $errors, 'warning' );

			// Show the back button to try and make the transaction again using other service
			echo '<button type="button" class="btn btn-primary" onclick="window.location.reload()">'.JText::_('PLG_SYSTEM_RSDIRECTORYIDEAL_BACK').'</button>';
			return;
		}
		
		// redirect to the proper service
		if (!empty($redirect_url))
		{
			JFactory::getApplication()->redirect($redirect_url);
		}
    }
        
    /**
     * Process the form.
     *
     * @access private
     */
    private function processForm()
    {
		if ( !$this->canRun() )
			return;

		$jinput = JFactory::getApplication()->input;

		$task = strtolower($jinput->getCmd('rsdir_plugin_task'));
		$hash = $jinput->get('hash', '', 'none');

		// Exit the function if the hash is empty.
		if ( empty($hash) )
		{
			return;
		}

		// Get the user transaction.
		$user_transaction = RSDirectoryHelper::getUserTransaction($hash);

		// Exit the function if no transaction was found.
		if (!$user_transaction)
		{
			return;
		}
		
		$redirect_url = $this->params->get('redirect_url');

		// The redirect_url parameter was not set.
		if (!$redirect_url)
		{
			$redirect_url = JRoute::_('index.php?option=com_rsdirectory&view=myaccount&tab=transactions', false);
		}
		
		// Initialize the log array.
		$log = array();
		
		if ($task == 'mollie.return')
		{
			try
			{
				$ideal   = RSDirectoryMollie::getInstance($this->params);
				$sale    = &$ideal->sale;

				$payment 		= $sale->payments->get($user_transaction->gateway_order_number);
				$isCancelled 	= $payment->isCanceled() || (!$payment->isPaid() && !$payment->isOpen());
				// Maybe aborted?
				if ($isCancelled && ($url = $this->params->get('mollie_cancelurl')))
				{
					$log[] = 'Service used Mollie';
					$log[] = 'Payment is canceled!';
					RSDirectoryHelper::saveTransactionLog($user_transaction->id, $log);
					JFactory::getApplication()->redirect($url);
				}
			}
			catch (Exception $e)
			{
				$log[] = 'Service used Mollie';
				$log[] = 'Service error encountered!';
				$log[] = $e->getMessage();
			}
			
			// Save the log
			if (!empty($log))
			{	
				RSDirectoryHelper::saveTransactionLog($user_transaction->id, $log);
			}
			
			JFactory::getApplication()->redirect($redirect_url);
			
			return;
		}
		
		// Exit the function if the transaction was already finalized.
		if ($user_transaction->status == 'finalized')
		{
			return;
		}

		//set the custom fields for the credit purchased email
		$email_custom_fields = array(
			'price-credits' 		=> $user_transaction->price,
			'tax' 					=> $user_transaction->tax,
			'tax-type' 				=> $user_transaction->tax_type,
			'tax-value' 			=> $user_transaction->tax_value,
			'total' 				=> $user_transaction->total,
			'credit-title' 			=> $user_transaction->credit_title,
			'currency' 				=> $user_transaction->currency,
			'credits' 				=> $user_transaction->credits,
			'payment-method' 		=> $user_transaction->gateway,
			'date-purchased' 		=> $user_transaction->date_created,
		);

		$log[] = "Receiving a new transaction from iDEAL.";

		// Get DBO.
		$db = JFactory::getDBO();

		$transaction_ok = false;
		$new_user_transaction = false;
		$send_mails = false;

		switch ($task)
		{
			case 'targetpay.notify':
				$send_mails = true;
				$log[] = 'Service used TargetPay';
				try
				{
					$ideal = RSDirectoryTargetPay::getInstance($this->params);
					$sale  = &$ideal->sale;

					
					$args = array(
						'trxid'       => $user_transaction->gateway_order_number
					);
					
					$sale->StatusRequest($args);

					if ($sale->status == 'Success')
					{
						$new_user_transaction = $this->save_payment($user_transaction);

						$log[] = "Successfully added payment to database.";
						$transaction_ok = true;
					}
					else
					{
						$log[] = 'Transaction did not pass.';
						throw new Exception(JText::_('PLG_SYSTEM_RSDIRECTORYIDEAL_PAYMENT_ERROR'));
					}
				}
				catch(Exception $e)
				{
					JFactory::getApplication()->enqueueMessage( $e->getMessage(), 'error' );
				}
			break;

			case 'sisow.notify':
				$send_mails = true;
				$log[] = 'Service used Sisow';
				try
				{
					$trxid = $jinput->getString('trxid', '');

					if ($trxid)
					{
						// Store the order number before checking the status
						$transaction_id = (object)array(
							'id' => $user_transaction->id,
							'gateway_order_number' => $trxid
						);

						$db->updateObject('#__rsdirectory_users_transactions', $transaction_id, 'id');
					}

					$ideal = RSDirectorySisow::getInstance($this->params);
					$sale  = &$ideal->sale;
					$sale->StatusRequest($trxid);

					if ($sale->status == 'Success')
					{
						$new_user_transaction = $this->save_payment($user_transaction);

						$log[] = "Successfully added payment to database.";
						$transaction_ok = true;
					}
					else
					{
						$log[] = 'Transaction did not pass.';
						throw new Exception(JText::_('PLG_SYSTEM_RSDIRECTORYIDEAL_PAYMENT_ERROR'));
					}
				}
				catch(Exception $e)
				{
					JFactory::getApplication()->enqueueMessage( $e->getMessage(), 'error' );
				}
			break;

			case 'mollie.notify':
				$send_mails = true;
				$log[] = 'Service used Mollie';
				try
				{
					/**
					 * Get an instance of the Mollie object so we can
					 * check the status of the transaction.
					 */
					$ideal = RSDirectoryMollie::getInstance($this->params);
					$sale = &$ideal->sale;
					$payment = $sale->payments->get($user_transaction->gateway_order_number);

					/**
					 * If it's paid, we update the database
					 */
					if ($payment->isPaid())
					{
						$new_user_transaction = $this->save_payment($user_transaction);

						$log[] = "Successfully added payment to database.";
						$transaction_ok = true;
					}
					else
					{
						$log[] = 'Transaction did not pass.';
						throw new Exception(JText::_('PLG_SYSTEM_RSDIRECTORYIDEAL_PAYMENT_ERROR'));
					}
				}
				catch(Exception $e)
				{
					JFactory::getApplication()->enqueueMessage( $e->getMessage(), 'error' );
				}
			break;
		}
		
		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/email.php';
		
		if ($send_mails && $transaction_ok && $new_user_transaction) {
			// set the email custom fields
			$email_custom_fields['gateway-order-number'] = $user_transaction->gateway_order_number;
			$email_custom_fields['gateway-order-type'] 	 = '';
			$email_custom_fields['date-finalized'] 		 = $new_user_transaction->date_finalized;
			$email_custom_fields['status'] 				 = JText::_('COM_RSDIRECTORY_TRANSACTION_STATUS_'.strtoupper($new_user_transaction->status));

			// Get entry.
			if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
				// Get the number of unpaid entry credits.
				$cost = RSDirectoryCredits::getUnpaidEntryCreditsSum($entry->id);

				// Get the total number of credits available to the user
				$user_credits = RSDirectoryCredits::getUserCredits($user_transaction->user_id);

				// Get plugin entry publishing param.
				$publish_entry = (int) $this->params->get('publish_entry', 0);

				// Verify if this entry can be published automatically.
				$automatically_publish_entry = $publish_entry == 1 || ($publish_entry == 2 && RSDirectoryHelper::checkUserPermission('auto_publish_entries', $user_transaction->user_id));

				// Check if enough credits are available.
				$has_enough_credits = $user_credits == 'unlimited' || $cost <= $user_credits;

				if ($automatically_publish_entry && $has_enough_credits) {
					// Holds emails that will be sent later on.
					$emails = array('credit_purchase_processed_finalized_with_entry');

					// Setup new entry data that should be overridden.
					$entry_data = (object)array(
						'id' => $entry->id,
						'paid' => 1,
						'published' => 1,
						'published_time' => JFactory::getDate()->toSql()
					);

					// Calculate expiry time.
					if ($entry->period) {
						$expiry_time = JFactory::getDate($entry_data->published_time);
						$expiry_time->modify("+{$entry->period} days");

						$entry_data->expiry_time = $expiry_time->toSql();
					} else {
						$entry_data->expiry_time = $db->getNullDate();
					}

					// Flag to send emails if the entry wasn't published and now it is.
					if (!$entry->published) {
						$emails[] = 'publish_entry';
					}

					// Subtract number of credits if the user doesn't have unlimited credits
					if ($user_credits != 'unlimited') {
						// Charge user.
						$query = $db->getQuery(true)
							->update($db->qn('#__rsdirectory_users'))
							->set($db->qn('credits') . ' = ' . $db->qn('credits') . ' - ' . $db->q($cost))
							->where($db->qn('user_id') . ' = ' . $db->q($user_transaction->user_id));

						$db->setQuery($query);
						$db->execute();
					}

					// Mark entry credits as being paid.
					$query = $db->getQuery(true)
						->update($db->qn('#__rsdirectory_entries_credits'))
						->set($db->qn('paid') . ' = ' . $db->q($entry_data->paid))
						->where($db->qn('entry_id') . ' = ' . $db->q($entry_data->id));

					$db->setQuery($query);
					$db->execute();

					// Save entry data.
					$db->updateObject('#__rsdirectory_entries', $entry_data, 'id');

					$log[] = 'Automatically published entry #'.$entry->id.'.';

					// Send emails.
					if ($emails) {
						// Get the form associated to the category.
						$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

						// Get the published form fields assigned to the form.
						$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

						// Send emails
						foreach ($emails as $email) {
							RSDirectoryEmail::getInstance($email, $form_fields, $entry, $form, ($email == 'credit_purchase_processed_finalized_with_entry' ? $email_custom_fields : null))->send();
							$log[] = "Sent $email.";
						}
					}
				} else {
					// Get the form associated to the category.
					$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

					// Get the published form fields assigned to the form.
					$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

					RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
					$log[] = "Sent credit_purchase_processed_finalized_with_entry.";
				}
			} else {
				$entry = (object) array('skip_entry'=>true, 'user_id' => $user_transaction->user_id);
				RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_no_entry', null, $entry, null, $email_custom_fields)->send();
				$log[] = "Sent credit_purchase_processed_finalized_no_entry.";
			}
		}
		else if ($send_mails && !$transaction_ok)
		{
			if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
				// Get the form associated to the category.
				$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

				// Get the published form fields assigned to the form.
				$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
			} else {
				$entry = (object)array('skip_entry' => true, 'user_id' => $user_transaction->user_id);
				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_no_entry', null, $entry, null, $email_custom_fields)->send();
			}
		}

		// Save the log
		RSDirectoryHelper::saveTransactionLog($user_transaction->id, $log);

		JFactory::getApplication()->redirect($redirect_url);
    }

	protected function save_payment($user_transaction)
	{
		$db = JFactory::getDBO();

		// Set the transaction params.
		$params = array();

		if (!empty($_POST)) {
			foreach ($_POST as $key => $value) {
				$params[] = "$key=$value";
			}
		}

		if (!empty($_GET)) {
			foreach ($_GET as $key => $value) {
				$params[] = "$key=$value";
			}
		}

		$new_user_transaction = (object)array(
			'id' => $user_transaction->id,
			'gateway_params' => implode("\n", $params),
			'date_finalized' => JFactory::getDate()->toSql(),
			'finalized' => 1,
			'status' => 'finalized',
		);

		$db->updateObject('#__rsdirectory_users_transactions', $new_user_transaction, 'id');

		RSDirectoryCredits::addUserCredits($user_transaction->user_id, $user_transaction->credits);

		return $new_user_transaction;
	}
		
	/**
     * Get taxa data.
     *
     * @access public
     *
     * @param object $vars
     *
     * @return array
     */
    public function onRsdirectorygetTaxData($vars)
    {
        if ( !$this->canRun() || !isset($vars->method) || !in_array($vars->method, self::$payment_methods) )
            return;
            
        // Calculate the tax.
        $tax_type = $this->params->get('tax_type');
		$tax_value = $this->params->get('tax_value');
        $tax = $tax_value && $tax_type ? $vars->price * ($tax_value/100) : $tax_value;
            
        $data = array(
            'tax_type' => $tax_type ? 'percent' : 'fixed',
            'tax_value' => $tax_value,
            'tax' => $tax,
        );
			
		return $data;
    }
}